{% extends "admin/base_site.html" %}
{% load admin_list i18n %}
{% load url from future %}
{% block extrastyle %}
  {{ block.super }}
  <link rel="stylesheet" type="text/css" href="{{ STATIC_URL }}admin/css/changelists.css" />
  {{ media.css }}
  {% if not actions_on_top and not actions_on_bottom %}
    <style>
      #changelist table thead th:first-child {width: inherit}
    </style>
  {% endif %}
{% endblock %}

{% block extrahead %}
{{ block.super }}
{{ media.js }}
{% if action_form %}{% if actions_on_top or actions_on_bottom %}
<script type="text/javascript">
(function($) {
    $(document).ready(function($) {
        $("tr input.action-select").actions();
    });
})(django.jQuery);
</script>
{% endif %}{% endif %}
{% endblock %}

{% block title %} Djrill Status | {% trans "Django site admin" %}{% endblock %}

{% block bodyclass %}change-list{% endblock %}

{% if not is_popup %}
  {% block breadcrumbs %}
    <div class="breadcrumbs">
      <a href="../../">
        {% trans "Home" %}
      </a>
       &rsaquo;
         Djrill
      &rsaquo;
      Status
    </div>
  {% endblock %}
{% endif %}

{% block coltype %}flex{% endblock %}

{% block content %}
  <div id="content-main">
    {% block object-tools %}
    {% endblock %}
    <div>
      {% block search %}{% endblock %}
      {% block date_hierarchy %}{% endblock %}

      {% block filters %}{% endblock %}
      {% block pagination %}{% endblock %}
      <dl>
          {% for term, value in status.items %}
          <dt>{{ term|capfirst }}</dt>
          <dd>{{ value }}</dd>
          {% endfor %}
      </dl>
    </div>
  </div>
{% endblock %}
