{% extends "admin/base_site.html" %}
{% load admin_list i18n %}
{% load url from future %}
{% load cycle from djrill_future %}

{% block extrastyle %}
  {{ block.super }}
  <link rel="stylesheet" type="text/css" href="{{ STATIC_URL }}admin/css/changelists.css" />
  {{ media.css }}
  {% if not actions_on_top and not actions_on_bottom %}
    <style>
      #changelist table thead th:first-child {width: inherit}
    </style>
  {% endif %}
{% endblock %}

{% block extrahead %}
{{ block.super }}
{{ media.js }}
{% endblock %}

{% block title %} Djrill Senders | {% trans "Django site admin" %}{% endblock %}

{% block bodyclass %}change-list{% endblock %}

{% if not is_popup %}
  {% block breadcrumbs %}
    <div class="breadcrumbs">
      <a href="../../">
        {% trans "Home" %}
      </a>
       &rsaquo;
         Djrill
      &rsaquo;
      Senders
    </div>
  {% endblock %}
{% endif %}

{% block coltype %}flex{% endblock %}

{% block content %}
  <div id="content-main">

    <div class="module filtered" id="changelist">
        {% block date_hierarchy %}{% endblock %}

      {% block filters %}
        {% include "djrill/_status.html" %}
      {% endblock %}

      {% block result_list %}
        {% if objects %}
        <div class="results">
            <table cellspacing="0" id="result_list">
                <thead>
                    <tr>
                        {% for header in objects.0.keys %}
                        <th scope="col">{{ header|capfirst }}</th>
                        {% endfor %}
                    </tr>
                </thead>
                <tbody>
                {% for result in objects %}
                    <tr class="{% cycle 'row1' 'row2' %}">
                        {% for item in result.values %}
                            <td>{{ item }}</td>
                        {% endfor %}
                    </tr>
                {% endfor %}
                </tbody>
            </table>
        </div>
        {% endif %}
      {% endblock %}
      {% block pagination %}{% endblock %}
    </div>
  </div>
{% endblock %}
