<div id="changelist-filter">
    <h2>Tools &amp; Info</h2>
    <h3>Status</h3>
    {% if status %}
        <p>Mandrill is <strong>UP</strong></p>
    {% else %}
        <p>Mandrill is <strong>DOWN</strong></p>
    {% endif %}
</div>
